// C Source File
// Created 3/15/01; 8:46:33 AM

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

short _ti89;                  // Produce .89Z File
short _ti92plus;              // Produce .9XZ File

enum ArrowKeys {UP,DOWN,LEFT,RIGHT};

#define	MOVE_RATE	5	// move sprite 5 pixels at a time
#define	HEIGHT		5	// sprites are 5 pixels tall

#define TI89_ESCROW	0xFFBF	// the ESC row on the TI-89 keyboard matrix
#define	TI89_ESCKEY	0x0001	// the ESC key inside that row

#define	TI92_ESCROW	0xFEFF	// the ESC row on the TI-92+ keyboard matrix
#define	TI92_ESCKEY	0x0040	// the ESC key inside that row

#define ARROW_ROW	0xFFFE	// arrow row is the same on both calculators

typedef struct {
	unsigned char x, y;
} POSITION;

typedef struct {
	unsigned char width, height;
} SCREEN;

static unsigned char block[] = {0xFF,0xFF,0xFF,0xFF,0xFF};

void getKeyMasks(short int *keys, short int calc) {
	// find the correct key masks based on which calculator we have
	if (calc == 0) {		// do we have a TI-89
		keys[0] = 0x0001;	// bit 0
		keys[1] = 0x0004;	// bit 2
		keys[2] = 0x0002;	// bit 1
		keys[3] = 0x0008;	// bit 3
	} else {			// then we must have a TI-92+
		keys[0] = 0x0020;	// bit 5
		keys[1] = 0x0080;	// bit 7
		keys[2] = 0x0010;	// bit 4
		keys[3] = 0x0040;	// bit 6
	}
}

inline void drawBlock(POSITION pos) {
	Sprite8(pos.x,pos.y,HEIGHT,block,LCD_MEM,SPRT_XOR);
}

// the name looks better, but with unmasked sprites, drawing and erasing are the same
inline void eraseBlock(POSITION pos) {
	drawBlock(pos);
}

// I think you can see why these are all declared 'inline'
inline void moveBlock(POSITION oldPosition, POSITION newPosition) {
	eraseBlock(oldPosition);
	drawBlock(newPosition);
}

short int quit(short int calc) {
	if (calc == 0) {	// test for TI-89 ESC key
		if (_rowread(TI89_ESCROW) & TI89_ESCKEY) {
			return 1;
		}
	} else {		// test for TI-92+ ESC key
		if (_rowread(TI92_ESCROW) & TI92_ESCKEY) {
			return 1;
		}
	}
	
	return 0;
}

void move(POSITION *newPosition, short int direction, const SCREEN screen) {
	POSITION oldPosition = *newPosition;
	
	switch (direction) {
		case UP:
			// if we can move up, then do so
			if ((newPosition->y - MOVE_RATE) > 0) {
				newPosition->y -= MOVE_RATE;
				moveBlock(oldPosition,*newPosition);
			}
			break;
		case DOWN:
			// if we can move down, then do so
			if ((newPosition->y + MOVE_RATE) < (screen.height - MOVE_RATE)) {
				newPosition->y += MOVE_RATE;
				moveBlock(oldPosition,*newPosition);
			}
			break;
		case LEFT:
			// if we can move left, then do so
			if ((newPosition->x - MOVE_RATE) > 0) {
				newPosition->x -= MOVE_RATE;
				moveBlock(oldPosition,*newPosition);
			}
			break;
		case RIGHT:
			// if we can move right, then do so
			if ((newPosition->x + MOVE_RATE) < (screen.width - MOVE_RATE)) {
				newPosition->x += MOVE_RATE;
				moveBlock(oldPosition,*newPosition);
			}
			break;
	}
}

// slow the program down
void delay(void) {
	short int loop = 1800, randNum;
	
	// generate random numbers to slow down the program...
	while (loop-- > 0) {
		randNum = rand() % loop;
	}
}

// Main Function
void _main(void) {
	short int keys[4];
	short int calc = CALCULATOR, key;
	INT_HANDLER interrupt1 = GetIntVec(AUTO_INT_1);	// save auto-interrupt 1
	POSITION pos = {0,0};
	SCREEN screen;
	
	// seed the random number generator
	randomize();
	
	// initialize the screen dimensions
	screen.width = LCD_WIDTH;
	screen.height = LCD_HEIGHT;
	
	// get the correct key masks based on which calculator we have. The TI-89
	// has a different keyboard mapping than the TI-92+
	getKeyMasks(keys,calc);
	
	// destroy auto-interrupt 1 so that it doesn't interfere with _rowread()
	SetIntVec(AUTO_INT_1,DUMMY_HANDLER);
	
	// clear the screen
	ClrScr();
	
	// draw the block on the screen at our initial position
	drawBlock(pos);
	
	// until the user presses ESC
	while (!quit(calc)) {
		key = _rowread(ARROW_ROW);
	
		// check for UP arrow
		if (key & keys[UP]) {
			move(&pos,UP,(const SCREEN)screen);
		}
		
		// check for LEFT arrow
		if (key & keys[LEFT]) {
			move(&pos,LEFT,(const SCREEN)screen);
		}
		
		// check for DOWN arrow
		if (key & keys[DOWN]) {
			move(&pos,DOWN,(const SCREEN)screen);
		}
		
		// check for RIGHT arrow
		if (key & keys[RIGHT]) {
			move(&pos,RIGHT,(const SCREEN)screen);
		}
		
		// slow the program down
		delay();
	}
	
	// restore auto interrupt 1
	SetIntVec(AUTO_INT_1,interrupt1);
	
	// wait for input before exiting the program
	ngetchx();
}